.rglPlayer {
  width: auto;
  height: auto;
}

.rglPlayer .rgl-button {
  width: auto;
  display: inline-block;
  font-size: 75%;
}

.rglPlayer .rgl-slider {
  display: inline-block;
  width: 30%;
}

.rglPlayer .rgl-label {
  display: inline;
  padding-left: 6px;
  padding-right: 6px;
}
